#include "io430.h"

    void PORT_Init()
    {
      //XT2 Oscilator PORTS
      P5SEL = P5SEL | 0x0C;                   //set XT2 pins as alternate function
      P5DIR = P5DIR | 0x0C;                   //set XIN/XOUT as output

      //Clock PORTS
      P11SEL = P11SEL | 0x07;                 //P11.0/P11.1/P11.2 set as alternate funcions
      P11DIR = P11DIR | 0x07;                 //P11.0/P11.1/P11.2 set as output
	
      //ADC PORTS
      P8SEL = P8SEL | 0x02;                   //PP8.1 --> ADC (ADC_OUT - remake) function
      P6SEL = P6SEL | 0x80;                   //P6.7 --> ADC function
      P8DIR = P8DIR | 0x02;                   //P8.1 (ADC_OUT - remake) set as output
      P6DIR = P6DIR & 0x7F;                   //PORT6.7 (ADC_IN)  set as input
    }

      void CLOCK_XT2_Init()
    {
      int timeout = 5000;
      P11DIR = P11DIR | 0x02 | 0x04;            //set P11.1 & P11.2 as output
      P11SEL = P11SEL | 0x02 | 0x04;            //set P11.1 & P11.2 as MCLK/SMCLK function
      P5SEL = P5SEL | 0x04 | 0x08;              //set P5.2 & P5.3 as XT2IN/XT2OUT

      UCSCTL6 = UCSCTL6 & (~XT2OFF);            //set XT2OFF=0 (bit8 = 0) --> enable XT2
      UCSCTL3 = UCSCTL3 | 0x20;                 //set SELREF = 010 --> FLL clock source = REFOCLK
      UCSCTL4 = UCSCTL4 | 0x200;                //set SELA = 010 --> ACLK=REFO

      do
      {
        if (!(UCSCTL7 & 0x08)) break;
        UCSCTL7 = UCSCTL7 & ~(0x0F);            // Clear XT2,XT1,DCO fault flags
        SFRIFG1 = SFRIFG1 & ~(0x02);            // Clear fault flags (SFRIFG1.OFIFG = 0)
        timeout--;
      }
      while ((SFRIFG1 & 0x02) && timeout);      // Test oscillator fault flag

      UCSCTL5 = UCSCTL5 | 0x0005;               //MCLK/SMCLK divisor = 32

      UCSCTL4 = UCSCTL4 | 0x55;                 //SMCLK=MCLK=XT2
      UCSCTL4 = ((UCSCTL4 & ~0x100) & ~0x200) & ~0x400;     //ACLK=XT1  (SELA = 0b000)
    }

    void ADC12_Init()
    {
      P8SEL = P8SEL | 0x02;                   //P8.1 --> ADC function
      P8DIR = P8DIR | 0x02;                   //PORT8.1 = 1 (ADC_OUT) set as output
      P6SEL = P6SEL | 0x80;                   //P6.7 --> ADC function
      P6DIR = P6DIR & ~0x80;                  //PORT6.7 = 0 (ADC_IN)  set as input

      ADC12CTL0 = ADC12CTL0 & ~(0x02);        //ADC12ENC = 0


      ADC12CTL0 = ADC12CTL0 | 1<<4;           //ADC12ON = 1
      ADC12CTL0 = ADC12CTL0 | 0x9900;         //ADC12SHT(0/1) = 0b1001 (Cycles = 384)

      ADC12CTL1 = ADC12CTL1 | 0x08 | 1<<4;    //ADC12SEL bits = 0b11 (Source clock - SMCLK)
      ADC12CTL1 = ADC12CTL1 & ~(0x02 | 0x04); //ADC12CONSEQ = 0b00 --> Single Chanel, Single Conversion
      ADC12CTL1 = ADC12CTL1 | 0x7000;         //select first (in order) register (7)
      ADC12CTL1 = ADC12CTL1 | 1<<9;           //SAMPCON - sampling timer

      ADC12CTL2 = ADC12CTL2 & ~(0x08);        //ADC12DF=0   --> unsigned format

      ADC12CTL2 = ADC12CTL2 & ~(1<<4 | 1<<5); //ADC12RES = 0b00 (8 bit mode resolution)

      ADC12MCTL7 = 0xF7;                      //End of Sequence --> Bit7 = 1; ADC12SREF = 0b111; ADC12INCH = 0b0111;


      ADC12CTL0 = ADC12CTL0 | 0x02;           //ADC12ENC = 1
    }

    unsigned int ADC12_Receive()
    {
      ADC12CTL0 = ADC12CTL0 | 0x01;           //start conversion ON
      while (ADC12CTL1 & 0x01);               //wait while BUSY
      return ADC12MEM7;
    }

    void TIMER_A_Init()
    {
      TA0CTL = (TA0CTL | 1<<9) & ~(1<<8);     //TimerA clock source is SMCLK (bit8=0, bit9=1)

      TA0CTL = (TA0CTL | 1<<4) & ~(1<<5);     //Mode Control set to UP mode (MC bits = 0b01)
      TA0CCTL0 = TA0CCTL0 | 0x00A0;           //OUTMODE0 = 0b001 (set mode)
      TA0CCTL1 = TA0CCTL1 | 0x00C0;           //OUTMODE1 = 0b110 (set/reset mode)

      TA0CCR0 = 0x00;                         //turn off timer
    }


int main(void)
{
  int del;
  // Stop watchdog timer to prevent time out reset
  WDTCTL = WDTPW + WDTHOLD;
  //__bis_SR_register(GIE);

  PORT_Init();
  CLOCK_XT2_Init();
  ADC12_Init();
  TIMER_A_Init();

  TA0CCR0 = 0xFF;
  //Endless loop
  while (1)
  {
	TA0CCR1 = ADC12_Receive();
	for (del=10; del; del--);
  }
}
